package org.apidesign.apifest08.currency;

import java.util.Currency;

/**
 * Abstract convertor implementation.
 * This class provide necessary stuff for creating any CurrencyConvertor
 * Convert value can be get by different way. Conversion should be made in its subclass.
 * @author Ladislav Vitasek
 */
abstract class AbstractCurrencyConvertor implements Convertor {
    protected final ConversionRatioProvider conversionRatioProvider;
    protected final Currency currency1;
    protected final Currency currency2;

    public AbstractCurrencyConvertor(Currency currency1, Currency currency2, ConversionRatioProvider conversionRatioProvider) {
        this.currency1 = currency1;
        this.currency2 = currency2;
        this.conversionRatioProvider = conversionRatioProvider;
    }

    
    public Currency getCurrency1() {
        return currency1;
    }


    public Currency getCurrency2() {
        return currency2;
    }

//    /**
//     * Conversion value can be get by Different way. Let's decide subclass where to get actual value
//     * @return current value of conversion constant
//     */
//    protected abstract BigDecimal getConversionValue();

    @SuppressWarnings({"RedundantIfStatement"})
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        AbstractCurrencyConvertor that = (AbstractCurrencyConvertor) o;

        if (!conversionRatioProvider.equals(that.conversionRatioProvider)) return false;
        if (!currency1.equals(that.currency1)) return false;
        if (!currency2.equals(that.currency2)) return false;

        return true;
    }

    public int hashCode() {
        int result;
        result = conversionRatioProvider.hashCode();
        result = 31 * result + currency1.hashCode();
        result = 31 * result + currency2.hashCode();
        return result;
    }

    ConversionRatioProvider getConversionRatioProvider() {
        return conversionRatioProvider;
    }    

}
